read_instance_stg <- function(filename) {
  require(igraph)
  require(purrr)
  require(readr)
  require(stringr)
  # Read lines
  con <- file(filename, open = "r")
  lines <- readLines(con)
  close(con)
  # Parse parameters
  info <- map(lines, ~ str_match(., "#([^0-9]*):(.*)")[2:3]) %>%
    discard(~ any(is.na(.))) # omit.na() see cheat sheet
  params <- list()
  for (i in info)
    params[str_replace_all(i[1], fixed(" "), "")] <- str_trim(i[2])
  # Build graph
  n <- parse_integer(lines[1]) + 2
  cost <- NULL
  edges <- list()
  for (i in seq_len(n)) {
    values <- parse_integer(str_match_all(lines[i + 1], "[0-9]+")[[1]][,1])
    cost <- c(cost, values[2])
    for (j in tail(values, -3))
      edges[[length(edges) + 1]] <- c(j + 1, i)
  }
  g <- make_empty_graph(n = n) %>%
    add.edges(unlist(edges)) %>%
    set_vertex_attr("cost", value = cost)
  list(graph = g, params = params)
}
