from mpmath import mpf
import random

def choose(n, k):
    """
    A fast way to calculate binomial coefficients by Andrew Dalke (contrib).
    """
    if 0 <= k <= n:
        ntok = 1
        ktok = 1
        for t in xrange(1, min(k, n - k) + 1):
            ntok *= n
            ktok *= t
            n -= 1
        return ntok // ktok
    else:
        return 0

def DAGCounting(n, verbose = False):
    """Precomputation for the recursive method.

    Keyword arguments:
    n -- maximum number of vertices
    """
    A = {}
    B = {}
    for size in range(1, n):
        B[(size, 1)] = mpf(0)
        A[(size, 1)] = mpf(0)
        A[size] = mpf(0)
    B[(1, 1)] = mpf(1)
    A[(1, 1)] = mpf(1)
    A[1] = mpf(1)
    for size in range(1, n + 1):
        if verbose:
            print size
        if size > 1:
            for sources in range(1, n):
                B[(size, sources)] = mpf(0)
                for s in range(1, size - sources + 1):
                    B[(size, sources)] += ((mpf(2) ** (sources) - 1) ** s) * \
                        mpf((2) ** (sources * (size - sources - s))) * \
                        A[(size - sources, s)]
                A[(size, sources)] = choose(size, sources) * B[(size, sources)]
        B[(size, size)] = mpf(1)
        A[(size, size)] = mpf(1)
        A[size] = mpf(0)
        for sources in range(1, size + 1):
            A[size] += A[(size, sources)]
    return A

def randomShapeUnif(n, k = 1):
    """Generate shapes uniformly on all DAGs.

    Keyword arguments:
    n -- maximum number of vertices
    k -- number of shapes to build
    """
    global A
    shapevector = []
    res = []
    i = n
    for nb in range(k):
        i = n
        shapevector = []
        while i > 0:
            x = random.random()
            somme = A[(i, 1)] / A[i]
            j = 1
            while x > somme:
                j = j + 1
                somme = somme + A[(i, j)] / A[i]
            shapevector.append(j)
            i = i - j
        res.append(shapevector)
    return res

def shapeToDAG(shape, verbose = False):
    """Generate a uniform DAG from a shape.

    Keyword arguments:
    shape -- partitions of the vertices
    """
    n = sum(shape)
    L = range(n)
    # distribute vertex labels
    res = []
    for k in shape:
        level = random.sample(L, k)
        for x in level:
            L.remove(x)
        res.append(level)
    if verbose:
        print res
    # add edges
    edge_list = []
    for i in range(1, len(shape)): # each level
        for v in res[i]: # each vertex in the level
            x = random.choice(res[i - 1]) # consider the previous level
            pred = [x]
            for u in res[i - 1]:
                if u != x and random.random() < 0.5:
                    pred.append(u)
            # all other previous levels
            for j in range(0, i - 1):
                for u in res[j]:
                    if random.random() < 0.5:
                        pred.append(u)
            for u in pred:
                edge_list.append((u, v))
    return edge_list

A = {}

def init(n, seed = -1):
    if seed != -1:
        random.seed(seed)
    global A
    A = DAGCounting(n = n)

def generateUniformDAG(n):
    shape = randomShapeUnif(n = n)[0]
    return shapeToDAG(shape)
